#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


"""Film identification tool
"""

import os
import sys
import re
import optparse

import rpcclient
import ident
import rating
import view


class AkyMovieIden(object):
    """Film identification class."""

    re_ext = re.compile(""".*\.(3g2|3gp|3gp2|3gpp|60d|ajp|asf|asx|avchd|avi|bik
                        |bix|box|cam|dat|divx|dmf|dv|dvr-ms|evo|flc|fli|flic
                        |flv|flx|gvi|gvp|h264|m1v|m2p|m2ts|m2v|m4e|m4v|mjp
                        |mjpeg|mjpg|mkv|moov|mov|movhd|movie|movx|mp4|mpe|mpeg
                        |mpg|mpv|mpv2|mxf|nsv|nut|ogg|ogm|omf|ps|qt|ram|rm|rmvb
                        |swf|ts|vfw|vid|video|viv|vivo|vob|vro|wm|wmv|wmx|wrap
                        |wvx|wx|x264|xvid)$""", re.VERBOSE | re.IGNORECASE)

    def __init__(self, out_view, verbose=False):
        self.verbose = verbose
        self.output = out_view
        self.rpc = rpcclient.OsRpcClient()
        self.movies = []
        
    def search_movies(self, path):
        """Searches for video files and identifies them."""
        for dirpath, _, files in os.walk(path):
            if (dirpath.rsplit('/', 1)[-1]).upper() == 'VIDEO_TS':
                continue   # DVD video parts ignore
            for filmfile in files:
                if re.match(self.re_ext, filmfile):
                    self.movies.append({'Path':dirpath, 'Filename':filmfile})
        ident.identify(self.movies, self.verbose)
        
    def get_rating(self):
        """Gets ratings for movies."""
        rating.rate_all(self.movies, self.verbose)

    def print_results(self):
        """Prints results."""
        if self.verbose:
            print
        self.output.print_out(self.movies)


if __name__ == '__main__':
    
    # Command-line argument parsing
    parser = optparse.OptionParser("%prog [options]")
    parser.add_option("--path", "-p", dest="path", default=os.getcwd(),
                        help="path to directory with movie/movies.")
    parser.add_option("--lang", "-l", dest="lang", default=None,
                        help="sets output report language.")
    parser.add_option("--info", "-i", dest="info", action='store_true', 
                        default=False,
                        help="prints list of avalible languages.")
    parser.add_option("--verbose", "-v", dest="verbose", action='store_true', 
                        default=False,
                        help="prints information about work progress.")
    opt, args = parser.parse_args()

    out_view = view.ViewStdout()

    # Prepare for job
    if opt.info:
        print "Available languages:"
        for lang in out_view.available_translation():
            print lang
        sys.exit(0)

    if not os.path.isdir(opt.path):
        sys.stderr.write('Directory "{0}" doesn\'t exist.\n'.format(opt.path))
        sys.exit(1)

    if opt.lang:
        if opt.lang in out_view.available_translation():
            try:
                out_view.set_translation(opt.lang)
            except (view.ViewError) as err:
                sys.stderr.write(str(err))
                sys.exit(1)
        else:
            sys.stderr.write('File for {0} doesn\'t exist.\n'.format(opt.lang))
            sys.exit(1)

    # Main functionality
    try:
        aky = AkyMovieIden(out_view, opt.verbose)
    except (rpcclient.RPCError) as err:
        sys.stderr.write(str(err))
        sys.exit(1)

    aky.search_movies(opt.path)
    aky.get_rating()    
    aky.print_results()
