#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


"""Module for RCP (XMLRPC) comunication with opensubtitles.org.
Contain singleton class: OsRpcClient
"""

import xmlrpclib
import socket

# Configuration
SERVER = "http://api.opensubtitles.org/xml-rpc"
USER = ""
PASS = ""
LANG = "cs"
USERAGENT = "aky0.1"


class RPCError(Exception):
    """Cannot open RPC connection."""
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return repr(self.msg)


class OsRpcClient(object):
    """Singleton class for RPC comunication with OpenSubtitles"""
    __state = {}
    counter = 0

    def __init__(self):
        self.__dict__ = self.__state
        if self.__dict__ == {}:
            try:
                self.client = xmlrpclib.ServerProxy(SERVER, allow_none=True)
            except (socket.error):
                raise RPCError('Connection reset by peer\n')
            login = self.client.LogIn(USER, PASS, LANG, USERAGENT)
            if login['status'] != '200 OK':
                raise RPCError('Cannot open RPC connection\n')
            self.token = login['token']
            self.counter = 0
        self.counter += 1
            

    def __del__(self):
        self.counter -= 1
        if not self.counter and self.__dict__:
            try:
                self.client.LogOut(self.token)
            except (socket.error):
                pass
   
         
    def movie_by_hash(self, mhash):
        """Searchs movie upon the recieved hash.
        More info:
        http://trac.opensubtitles.org/projects/opensubtitles/wiki/XMLRPC
        """
        if not mhash:
            return {}

        try:
            res = self.client.CheckMovieHash(self.token, [mhash])['data'][mhash]
        except (KeyError, socket.error):
            return {}
        return res


    def movie_by_imdbid(self, imdbid):
        """Searchs movie upon recieved imdb id."""
        try:
            info = self.client.GetIMDBMovieDetails(self.token, imdbid)['data']
        except (KeyError, socket.error):
            return {}

        if not info:
            return {}
        brief_info = {}
        brief_info['MovieImdbID'] = imdbid
        brief_info['MovieName'] = info['title']
        brief_info['MovieHash'] = ''
        brief_info['MovieYear'] = ('year' in info and info['year']) or None    
        return brief_info


    def movie_rating_by_imdbid(self, imdbid):
        """Gets movie IMDB rating upon recieved imdb id."""
        try:
            info = self.client.GetIMDBMovieDetails(self.token, imdbid)['data']
        except (KeyError, socket.error):
            return None

        if info and 'rating' in info and info['rating']:
             return info['rating']
        return None


    def movie_by_name(self, name):
        """Searchs movie details by name on IMDb."""
        try:
            imdbid = self.client.SearchMoviesOnIMDB(self.token, name)
            imdbid = imdbid['data'][0]['id']
        except (IndexError, KeyError, TypeError, socket.error):
            return {}
        return self.movie_by_imdbid(imdbid)


if __name__ == '__main__':
    client = OsRpcClient()
